@@ -1,6 +1,6 @@
 import logging
 import numbers
-from typing import List
+from typing import Optional, List
 
 import numpy as np
 import torch
@@ -12,7 +12,9 @@ from haystack.modeling.utils import flatten_list
 logger = logging.getLogger(__name__)
 
 
-def flatten_rename(encoded_batch: BatchEncoding, keys: List[str] = None, renamed_keys: List[str] = None):
+def flatten_rename(
+    encoded_batch: BatchEncoding, keys: Optional[List[str]] = None, renamed_keys: Optional[List[str]] = None
+):
     if encoded_batch is None:
         return []
     if not keys:
