@@ -376,7 +376,7 @@ class BiAdaptiveModel(nn.Module):
         try:
             tracker.track_params(params)
         except Exception as e:
-            logger.warning(f"ML logging didn't work: {e}")
+            logger.warning("ML logging didn't work: %s", e)
 
     def verify_vocab_size(self, vocab_size1: int, vocab_size2: int):
         """
