@@ -1,5 +1,6 @@
 from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Type, Union
 
+import os
 import sys
 import json
 import inspect
@@ -24,11 +25,10 @@ from haystack.nodes.base import BaseComponent
 
 
 logger = logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
 
 
 JSON_SCHEMAS_PATH = Path(__file__).parent.parent.parent / "haystack" / "json-schemas"
-SCHEMA_URL = "https://raw.githubusercontent.com/deepset-ai/haystack/main/haystack/json-schemas/"
+SCHEMA_URL = "https://raw.githubusercontent.com/deepset-ai/haystack-json-schema/main/json-schema/"
 
 # Allows accessory classes (like enums and helpers) to be registered as valid input for
 # custom node's init parameters. For now we disable this feature, but flipping this variables
@@ -147,7 +147,7 @@ def handle_optional_params(param_fields: List[inspect.Parameter], params_schema:
     for param in optional_params:
         param_dict = params_schema["properties"][param.name]
         type_ = param_dict.pop("type", None)
-        if type_:
+        if type_ is not None:
             if "items" in param_dict:
                 items = param_dict.pop("items")
                 param_dict["anyOf"] = [{"type": type_, "items": items}, {"type": "null"}]
@@ -155,7 +155,8 @@ def handle_optional_params(param_fields: List[inspect.Parameter], params_schema:
                 param_dict["anyOf"] = [{"type": type_}, {"type": "null"}]
         else:
             anyof_list = param_dict.pop("anyOf", None)
-            if anyof_list:
+            if anyof_list is not None:
+                anyof_list = list(sorted(anyof_list, key=lambda x: x["type"]))
                 anyof_list.append({"type": "null"})
                 param_dict["anyOf"] = anyof_list
     return params_schema
@@ -176,7 +177,7 @@ def create_schema_for_node_class(node_class: Type[BaseComponent]) -> Tuple[Dict[
 
     node_name = getattr(node_class, "__name__")
 
-    logger.info("Creating schema for '%s'", node_name)
+    logger.debug("Creating schema for '%s'", node_name)
 
     # Read the relevant init parameters from __init__'s signature
     init_method = getattr(node_class, "__init__", None)
@@ -380,6 +381,7 @@ def get_json_schema(filename: str, version: str, modules: List[str] = ["haystack
         ],
         "definitions": schema_definitions,
     }
+
     return pipeline_schema
 
 
@@ -404,36 +406,56 @@ def inject_definition_in_schema(node_class: Type[BaseComponent], schema: Dict[st
     return schema
 
 
-def update_json_schema(destination_path: Path = JSON_SCHEMAS_PATH):
+def load_schema():
     """
-    If the version contains "rc", only update main's schema.
-    Otherwise, create (or update) a new schema.
+    Generate the json schema if it doesn't exist and load it
+    """
+    schema_file_path = JSON_SCHEMAS_PATH / "haystack-pipeline-main.schema.json"
+    if not os.path.exists(schema_file_path):
+        logging.info("Json schema not found, generating one at: %s", schema_file_path)
+        try:
+            update_json_schema(main_only=True)
+        except Exception as e:
+            # Be sure not to remain with an empty file if something went wrong
+            if schema_file_path.exists():
+                schema_file_path.unlink()
+            # This error is not recoverable
+            raise e
+
+    with open(schema_file_path, "r") as schema_file:
+        return json.load(schema_file)
+
+
+def update_json_schema(destination_path: Path = JSON_SCHEMAS_PATH, main_only: bool = False):
+    """
+    Create (or update) a new schema.
     """
     # `main` schema is always updated and will contain the same data as the latest
     # commit from `main` or a release branch
     filename = f"haystack-pipeline-main.schema.json"
+
+    os.makedirs(destination_path, exist_ok=True)
     with open(destination_path / filename, "w") as json_file:
         json.dump(get_json_schema(filename=filename, version="ignore"), json_file, indent=2)
 
-    # Create/update the specific version file too
-    filename = f"haystack-pipeline-{haystack_version}.schema.json"
-    with open(destination_path / filename, "w") as json_file:
-        json.dump(get_json_schema(filename=filename, version=haystack_version), json_file, indent=2)
+    if not main_only and "rc" not in haystack_version:
 
-    # Update the index
-    index_name = "haystack-pipeline.schema.json"
-    with open(destination_path / index_name, "r") as json_file:
-        index = json.load(json_file)
-        new_entry = {
-            "allOf": [
-                {"properties": {"version": {"const": haystack_version}}},
-                {
-                    "$ref": "https://raw.githubusercontent.com/deepset-ai/haystack/main/haystack/json-schemas/"
-                    f"haystack-pipeline-{haystack_version}.schema.json"
-                },
-            ]
-        }
-        if new_entry not in index["oneOf"]:
-            index["oneOf"].append(new_entry)
-    with open(destination_path / index_name, "w") as json_file:
-        json.dump(index, json_file, indent=2)
+        # Create/update the specific version file too
+        filename = f"haystack-pipeline-{haystack_version}.schema.json"
+        with open(destination_path / filename, "w") as json_file:
+            json.dump(get_json_schema(filename=filename, version=haystack_version), json_file, indent=2)
+
+        # Update the index
+        index_name = "haystack-pipeline.schema.json"
+        with open(destination_path / index_name, "r") as json_file:
+            index = json.load(json_file)
+            new_entry = {
+                "allOf": [
+                    {"properties": {"version": {"const": haystack_version}}},
+                    {"$ref": f"{SCHEMA_URL}haystack-pipeline-{haystack_version}.schema.json"},
+                ]
+            }
+            if new_entry not in index["oneOf"]:
+                index["oneOf"].append(new_entry)
+        with open(destination_path / index_name, "w") as json_file:
+            json.dump(obj=index, fp=json_file, indent=2, sort_keys=True)
