@@ -3,6 +3,7 @@ from typing import List, Optional, Union, Dict, Any, Tuple
 import logging
 import itertools
 
+import torch
 from transformers import pipeline
 from transformers.data.processors.squad import SquadExample
 
@@ -37,6 +38,7 @@ class TransformersReader(BaseReader):
         doc_stride: int = 128,
         batch_size: int = 16,
         use_auth_token: Optional[Union[str, bool]] = None,
+        devices: Optional[List[Union[str, torch.device]]] = None,
     ):
         """
         Load a QA model from Transformers.
@@ -72,16 +74,27 @@ class TransformersReader(BaseReader):
                                `transformers-cli login` (stored in ~/.huggingface) will be used.
                                Additional information can be found here
                                https://huggingface.co/transformers/main_classes/model.html#transformers.PreTrainedModel.from_pretrained
+
+        :param devices: List of torch devices (e.g. cuda, cpu, mps) to limit inference to specific devices.
+                        A list containing torch device objects and/or strings is supported (For example
+                        [torch.device('cuda:0'), "mps", "cuda:1"]). When specifying `use_gpu=False` the devices
+                        parameter is not used and a single cpu device is used for inference.
         """
         super().__init__()
 
-        self.devices, _ = initialize_device_settings(use_cuda=use_gpu, multi_gpu=False)
-        device = 0 if self.devices[0].type == "cuda" else -1
+        self.devices, _ = initialize_device_settings(devices=devices, use_cuda=use_gpu, multi_gpu=False)
+
+        if len(self.devices) > 1:
+            logger.warning(
+                f"Multiple devices are not supported in {self.__class__.__name__} inference, "
+                f"using the first device {self.devices[0]}."
+            )
+
         self.model = pipeline(
             "question-answering",
             model=model_name_or_path,
             tokenizer=tokenizer,
-            device=device,
+            device=self.devices[0],
             revision=model_version,
             use_auth_token=use_auth_token,
         )
