@@ -6,7 +6,6 @@ import logging
 import torch
 from tqdm.auto import tqdm
 from transformers import pipeline
-from transformers.models.auto.modeling_auto import AutoModelForSeq2SeqLM
 
 from haystack.schema import Document
 from haystack.nodes.summarizer.base import BaseSummarizer
@@ -34,19 +33,18 @@ class TransformersSummarizer(BaseSummarizer):
     |
     |     # Summarize
     |     summary = summarizer.predict(
-    |        documents=docs,
-    |        generate_single_summary=True
-    |     )
+    |        documents=docs)
     |
-    |     # Show results (List of Documents, containing summary and original text)
+    |     # Show results (List of Documents, containing summary and original content)
     |     print(summary)
     |
     |    [
     |      {
-    |        "text": "California's largest electricity provider has turned off power to hundreds of thousands of customers.",
+    |        "content": "PGE stated it scheduled the blackouts in response to forecasts for high winds amid dry conditions. ...",
     |        ...
     |        "meta": {
-    |          "context": "PGE stated it scheduled the blackouts in response to forecasts for high winds amid dry conditions. ..."
+    |                   "summary": "California's largest electricity provider has turned off power to hundreds of thousands of customers.",
+    |                   ...
     |              },
     |        ...
     |      },
@@ -83,12 +81,9 @@ class TransformersSummarizer(BaseSummarizer):
         :param min_length: Minimum length of summarized text
         :param use_gpu: Whether to use GPU (if available).
         :param clean_up_tokenization_spaces: Whether or not to clean up the potential extra spaces in the text output
-        :param separator_for_single_summary: If `generate_single_summary=True` in `predict()`, we need to join all docs
-                                             into a single text. This separator appears between those subsequent docs.
-        :param generate_single_summary: Whether to generate a single summary for all documents or one summary per document.
-                                        If set to "True", all docs will be joined to a single string that will then
-                                        be summarized.
-                                        Important: The summary will depend on the order of the supplied documents!
+        :param separator_for_single_summary: This parameter is deprecated and will be removed in Haystack 1.12
+        :param generate_single_summary: This parameter is deprecated and will be removed in Haystack 1.12.
+                                        To obtain single summaries from multiple documents, consider using the [DocumentMerger](https://docs.haystack.deepset.ai/docs/document_merger).
         :param batch_size: Number of documents to process at a time.
         :param progress_bar: Whether to show a progress bar.
         :param use_auth_token: The API token used to download private models from Huggingface.
@@ -103,6 +98,11 @@ class TransformersSummarizer(BaseSummarizer):
         """
         super().__init__()
 
+        if generate_single_summary is True:
+            raise ValueError(
+                "'generate_single_summary' has been removed. Instead, you can use the Document Merger to merge documents before applying the Summarizer."
+            )
+
         self.devices, _ = initialize_device_settings(devices=devices, use_cuda=use_gpu, multi_gpu=False)
         if len(self.devices) > 1:
             logger.warning(
@@ -110,20 +110,20 @@ class TransformersSummarizer(BaseSummarizer):
                 f"using the first device {self.devices[0]}."
             )
 
-        # TODO AutoModelForSeq2SeqLM is only necessary with transformers==4.1.1, with newer versions use the pipeline directly
         if tokenizer is None:
             tokenizer = model_name_or_path
-        model = AutoModelForSeq2SeqLM.from_pretrained(
-            pretrained_model_name_or_path=model_name_or_path, revision=model_version, use_auth_token=use_auth_token
-        )
+
         self.summarizer = pipeline(
-            "summarization", model=model, tokenizer=tokenizer, device=self.devices[0], use_auth_token=use_auth_token
+            task="summarization",
+            model=model_name_or_path,
+            tokenizer=tokenizer,
+            revision=model_version,
+            device=self.devices[0],
+            use_auth_token=use_auth_token,
         )
         self.max_length = max_length
         self.min_length = min_length
         self.clean_up_tokenization_spaces = clean_up_tokenization_spaces
-        self.separator_for_single_summary = separator_for_single_summary
-        self.generate_single_summary = generate_single_summary
         self.print_log: Set[str] = set()
         self.batch_size = batch_size
         self.progress_bar = progress_bar
@@ -134,29 +134,23 @@ class TransformersSummarizer(BaseSummarizer):
         These document can for example be retrieved via the Retriever.
 
         :param documents: Related documents (e.g. coming from a retriever) that the answer shall be conditioned on.
-        :param generate_single_summary: Whether to generate a single summary for all documents or one summary per document.
-                                        If set to "True", all docs will be joined to a single string that will then
-                                        be summarized.
-                                        Important: The summary will depend on the order of the supplied documents!
-        :return: List of Documents, where Document.text contains the summarization and Document.meta["context"]
-                 the original, not summarized text
+        :param generate_single_summary: This parameter is deprecated and will be removed in Haystack 1.12.
+                                        To obtain single summaries from multiple documents, consider using the [DocumentMerger](https://docs.haystack.deepset.ai/docs/document_merger).
+        :return: List of Documents, where Document.meta["summary"] contains the summarization
         """
+        if generate_single_summary is True:
+            raise ValueError(
+                "'generate_single_summary' has been removed. Instead, you can use the Document Merger to merge documents before applying the Summarizer."
+            )
+
         if self.min_length > self.max_length:
             raise AttributeError("min_length cannot be greater than max_length")
 
         if len(documents) == 0:
             raise AttributeError("Summarizer needs at least one document to produce a summary.")
 
-        if generate_single_summary is None:
-            generate_single_summary = self.generate_single_summary
-
         contexts: List[str] = [doc.content for doc in documents]
 
-        if generate_single_summary:
-            # Documents order is very important to produce summary.
-            # Different order of same documents produce different summary.
-            contexts = [self.separator_for_single_summary.join(contexts)]
-
         encoded_input = self.summarizer.tokenizer(contexts, verbose=False)
         for input_id in encoded_input["input_ids"]:
             tokens_count: int = len(input_id)
@@ -182,15 +176,9 @@ class TransformersSummarizer(BaseSummarizer):
 
         result: List[Document] = []
 
-        if generate_single_summary:
-            for context, summarized_answer in zip(contexts, summaries):
-                cur_doc = Document(content=summarized_answer["summary_text"], meta={"context": context})
-                result.append(cur_doc)
-        else:
-            for context, summarized_answer, document in zip(contexts, summaries, documents):
-                cur_doc = Document(content=summarized_answer["summary_text"], meta=document.meta)
-                cur_doc.meta.update({"context": context})
-                result.append(cur_doc)
+        for summary, document in zip(summaries, documents):
+            document.meta.update({"summary": summary["summary_text"]})
+            result.append(document)
 
         return result
 
@@ -206,13 +194,14 @@ class TransformersSummarizer(BaseSummarizer):
 
         :param documents: Single list of related documents or list of lists of related documents
                           (e.g. coming from a retriever) that the answer shall be conditioned on.
-        :param generate_single_summary: Whether to generate a single summary for each provided document list or
-                                        one summary per document.
-                                        If set to "True", all docs of a document list will be joined to a single string
-                                        that will then be summarized.
-                                        Important: The summary will depend on the order of the supplied documents!
+        :param generate_single_summary: This parameter is deprecated and will be removed in Haystack 1.12.
+                                        To obtain single summaries from multiple documents, consider using the [DocumentMerger](https://docs.haystack.deepset.ai/docs/document_merger).
         :param batch_size: Number of Documents to process at a time.
         """
+        if generate_single_summary is True:
+            raise ValueError(
+                "'generate_single_summary' has been removed. Instead, you can use the Document Merger to merge documents before applying the Summarizer."
+            )
 
         if self.min_length > self.max_length:
             raise AttributeError("min_length cannot be greater than max_length")
@@ -225,14 +214,8 @@ class TransformersSummarizer(BaseSummarizer):
         if batch_size is None:
             batch_size = self.batch_size
 
-        if generate_single_summary is None:
-            generate_single_summary = self.generate_single_summary
-
-        single_doc_list = False
-        if isinstance(documents[0], Document):
-            single_doc_list = True
-
-        if single_doc_list:
+        is_doclist_flat = isinstance(documents[0], Document)
+        if is_doclist_flat:
             contexts = [doc.content for doc in documents if isinstance(doc, Document)]
         else:
             contexts = [
@@ -240,19 +223,8 @@ class TransformersSummarizer(BaseSummarizer):
                 for docs in documents
                 if isinstance(docs, list)
             ]
-
-        if generate_single_summary:
-            if single_doc_list:
-                contexts = [self.separator_for_single_summary.join(contexts)]
-            else:
-                contexts = [self.separator_for_single_summary.join(context_group) for context_group in contexts]
-            number_of_docs = [1 for _ in contexts]
-        else:
-            if single_doc_list:
-                number_of_docs = [1 for _ in contexts]
-            else:
-                number_of_docs = [len(context_group) for context_group in contexts]
-                contexts = list(itertools.chain.from_iterable(contexts))
+            number_of_docs = [len(context_group) for context_group in contexts]
+            contexts = list(itertools.chain.from_iterable(contexts))
 
         encoded_input = self.summarizer.tokenizer(contexts, verbose=False)
         for input_id in encoded_input["input_ids"]:
@@ -286,26 +258,30 @@ class TransformersSummarizer(BaseSummarizer):
         ):
             summaries.extend(summary_batch)
 
-        # Group summaries together
-        grouped_summaries = []
-        grouped_contexts = []
-        left_idx = 0
-        right_idx = 0
-        for number in number_of_docs:
-            right_idx = left_idx + number
-            grouped_summaries.append(summaries[left_idx:right_idx])
-            grouped_contexts.append(contexts[left_idx:right_idx])
-            left_idx = right_idx
+        if is_doclist_flat:
+            flat_result: List[Document] = []
+            flat_doc_list: List[Document] = [doc for doc in documents if isinstance(doc, Document)]
+            for summary, document in zip(summaries, flat_doc_list):
+                document.meta.update({"summary": summary["summary_text"]})
+                flat_result.append(document)
+            return flat_result
+        else:
+            nested_result: List[List[Document]] = []
+            nested_doc_list: List[List[Document]] = [lst for lst in documents if isinstance(lst, list)]
 
-        result = []
-        for summary_group, context_group in zip(grouped_summaries, grouped_contexts):
-            cur_summaries = [
-                Document(content=summary["summary_text"], meta={"context": context})
-                for summary, context in zip(summary_group, context_group)
-            ]
-            if single_doc_list:
-                result.append(cur_summaries[0])
-            else:
-                result.append(cur_summaries)  # type: ignore
+            # Group summaries together
+            grouped_summaries = []
+            left_idx = 0
+            right_idx = 0
+            for number in number_of_docs:
+                right_idx = left_idx + number
+                grouped_summaries.append(summaries[left_idx:right_idx])
+                left_idx = right_idx
 
-        return result
+            for summary_group, docs_group in zip(grouped_summaries, nested_doc_list):
+                cur_summaries = []
+                for summary, document in zip(summary_group, docs_group):
+                    document.meta.update({"summary": summary["summary_text"]})
+                    cur_summaries.append(document)
+                nested_result.append(cur_summaries)
+            return nested_result
