@@ -11,12 +11,13 @@ import requests
 logger = logging.getLogger(__name__)
 ELASTICSEARCH_CONTAINER_NAME = "elasticsearch"
 OPENSEARCH_CONTAINER_NAME = "opensearch"
-MILVUS1_CONTAINER_NAME = "milvus1"
 WEAVIATE_CONTAINER_NAME = "weaviate"
 
 
 def launch_es(sleep=15, delete_existing=False):
-    # Start an Elasticsearch server via Docker
+    """
+    Start an Elasticsearch server via Docker.
+    """
 
     logger.debug("Starting Elasticsearch ...")
     if delete_existing:
@@ -37,8 +38,9 @@ def launch_es(sleep=15, delete_existing=False):
 
 
 def launch_opensearch(sleep=15, delete_existing=False):
-    # Start an OpenSearch server via docker
-
+    """
+    Start an OpenSearch server via Docker.
+    """
     logger.debug("Starting OpenSearch...")
     # This line is needed since it is not possible to start a new docker container with the name opensearch if there is a stopped image with the same now
     # docker rm only succeeds if the container is stopped, not if it is running
@@ -60,12 +62,14 @@ def launch_opensearch(sleep=15, delete_existing=False):
 
 
 def launch_weaviate(sleep=15):
-    # Start a Weaviate server via Docker
+    """
+    Start a Weaviate server via Docker.
+    """
 
     logger.debug("Starting Weaviate ...")
     status = subprocess.run(
         [
-            f"docker start {WEAVIATE_CONTAINER_NAME} > /dev/null 2>&1 || docker run -d -p 8080:8080 --env AUTHENTICATION_ANONYMOUS_ACCESS_ENABLED='true' --env PERSISTENCE_DATA_PATH='/var/lib/weaviate' --name {WEAVIATE_CONTAINER_NAME} semitechnologies/weaviate:1.14.0"
+            f"docker start {WEAVIATE_CONTAINER_NAME} > /dev/null 2>&1 || docker run -d -p 8080:8080 --env AUTHENTICATION_ANONYMOUS_ACCESS_ENABLED='true' --env PERSISTENCE_DATA_PATH='/var/lib/weaviate' --name {WEAVIATE_CONTAINER_NAME} semitechnologies/weaviate:latest"
         ],
         shell=True,
     )
@@ -98,10 +102,6 @@ def stop_elasticsearch(delete_container=False):
     stop_container(ELASTICSEARCH_CONTAINER_NAME, delete_container)
 
 
-def stop_milvus(delete_container=False):
-    stop_container(MILVUS1_CONTAINER_NAME, delete_container)
-
-
 def stop_weaviate(delete_container=False):
     stop_container(WEAVIATE_CONTAINER_NAME, delete_container)
 
@@ -112,8 +112,6 @@ def stop_service(document_store, delete_container=False):
         stop_opensearch(delete_container)
     elif "ElasticsearchDocumentStore" in ds_class:
         stop_elasticsearch(delete_container)
-    elif "MilvusDocumentStore" in ds_class:
-        stop_milvus(delete_container)
     elif "WeaviateDocumentStore" in ds_class:
         stop_weaviate(delete_container)
     else:
@@ -121,8 +119,9 @@ def stop_service(document_store, delete_container=False):
 
 
 def launch_milvus(sleep=15, delete_existing=False):
-    # Start a Milvus server via docker
-
+    """
+    Start a Milvus server via Docker
+    """
     logger.debug("Starting Milvus ...")
 
     milvus_dir = Path.home() / "milvus"
@@ -143,31 +142,3 @@ def launch_milvus(sleep=15, delete_existing=False):
         )
     else:
         time.sleep(sleep)
-
-
-def launch_milvus1(sleep=15):
-    # Start a Milvus (version <2.0.0) server via docker
-
-    logger.debug("Starting Milvus ...")
-    logger.warning(
-        "Automatic Milvus config creation not yet implemented. "
-        "If you are starting Milvus using launch_milvus(), "
-        "make sure you have a properly populated milvus/conf folder. "
-        "See (https://milvus.io/docs/v1.0.0/milvus_docker-cpu.md) for more details."
-    )
-    status = subprocess.run(
-        [
-            f"docker start {MILVUS1_CONTAINER_NAME} > /dev/null 2>&1 || docker run -d --name {MILVUS1_CONTAINER_NAME} \
-          -p 19530:19530 \
-          -p 19121:19121 \
-          milvusdb/milvus:1.1.0-cpu-d050721-5e559c"
-        ],
-        shell=True,
-    )
-    if status.returncode:
-        logger.warning(
-            "Tried to start Milvus through Docker but this failed. "
-            "It is likely that there is already an existing Milvus instance running. "
-        )
-    else:
-        time.sleep(sleep)
